@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Quick 3D.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "qquick3dgeometry_p.h"
 #include "qquick3dscenemanager_p.h"
@@ -586,12 +560,103 @@ void QQuick3DGeometry::clear()
     d->m_vertexBuffer.clear();
     d->m_indexBuffer.clear();
     d->m_attributeCount = 0;
+    d->m_subsets.clear();
     d->m_primitiveType = PrimitiveType::Triangles;
     d->m_geometryChanged = true;
     d->m_min = {};
     d->m_max = {};
 }
 
+/*!
+    Returns the number of subsets.
+*/
+int QQuick3DGeometry::subsetCount() const
+{
+    Q_D(const QQuick3DGeometry);
+    return d->m_subsets.size();
+}
+
+/*!
+    Returns the number of minimum bounds of a \a subset.
+
+    \sa subsetBoundsMax
+*/
+QVector3D QQuick3DGeometry::subsetBoundsMin(int subset) const
+{
+    Q_D(const QQuick3DGeometry);
+    if (subset >= 0 && subset < d->m_subsets.size())
+        return d->m_subsets[subset].boundsMin;
+    return {};
+}
+
+/*!
+    Returns the number of maximum bounds of a \a subset.
+
+    \sa subsetBoundsMin
+*/
+QVector3D QQuick3DGeometry::subsetBoundsMax(int subset) const
+{
+    Q_D(const QQuick3DGeometry);
+    if (subset >= 0 && subset < d->m_subsets.size())
+        return d->m_subsets[subset].boundsMax;
+    return {};
+}
+
+/*!
+    Returns the \a subset offset to the vertex or index buffer.
+
+    \sa subsetCount
+*/
+int QQuick3DGeometry::subsetOffset(int subset) const
+{
+    Q_D(const QQuick3DGeometry);
+    if (subset >= 0 && subset < d->m_subsets.size())
+        return d->m_subsets[subset].offset;
+    return 0;
+}
+
+/*!
+    Returns the subset primitive count.
+
+    \sa subsetOffset
+*/
+int QQuick3DGeometry::subsetCount(int subset) const
+{
+    Q_D(const QQuick3DGeometry);
+    if (subset >= 0 && subset < d->m_subsets.size())
+        return d->m_subsets[subset].count;
+    return 0;
+}
+
+/*!
+    Returns the \a subset name.
+*/
+QString QQuick3DGeometry::subsetName(int subset) const
+{
+    Q_D(const QQuick3DGeometry);
+    if (subset >= 0 && subset < d->m_subsets.size())
+        return d->m_subsets[subset].name;
+    return {};
+}
+
+/*!
+    Adds new subset to the geometry. Subsets allow rendering parts of the geometry with different
+    materials. The materials are specified in the \l {Model::materials}{model}.
+
+    If the geometry has index buffer, then the \a offset and \a count are the primitive offset and
+    count of indices in the subset. If the geometry has only vertex buffer,
+    the offset is the vertex offset and count is the number of vertices in the subset.
+
+    The bounds \a boundsMin and \a boundsMax should enclose the subset just like geometry bounds.
+    Also the subset can have a \a name.
+*/
+void QQuick3DGeometry::addSubset(int offset, int count, const QVector3D &boundsMin, const QVector3D &boundsMax, const QString &name)
+{
+    Q_D(QQuick3DGeometry);
+    d->m_subsets.append({name, boundsMin, boundsMax, quint32(offset), quint32(count)});
+    d->m_geometryChanged = true;
+}
+
 static inline QSSGMesh::Mesh::DrawMode mapPrimitiveType(QQuick3DGeometry::PrimitiveType t)
 {
     switch (t) {
