@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Quick 3D.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 #include "qquick3dinstancing_p.h"
 #include "qquick3dscenemanager_p.h"
 #include <QtQuick3DRuntimeRender/private/qssgrenderinstancetable_p.h>
@@ -187,6 +161,136 @@ bool QQuick3DInstancing::depthSortingEnabled() const
     return d->m_depthSortingEnabled;
 }
 
+const QQuick3DInstancing::InstanceTableEntry *QQuick3DInstancing::getInstanceEntry(int index)
+{
+    const QByteArray data = getInstanceBuffer(nullptr);
+    if (index >= int(data.size() / sizeof(InstanceTableEntry)))
+        return nullptr;
+    return reinterpret_cast<const QQuick3DInstancing::InstanceTableEntry*>(data.constData()) + index;
+}
+
+QVector3D QQuick3DInstancing::InstanceTableEntry::getPosition() const
+{
+    return QVector3D{ row0[3], row1[3], row2[3] };
+}
+
+QVector3D QQuick3DInstancing::InstanceTableEntry::getScale() const
+{
+    const QVector3D col0{row0[0], row1[0], row2[0]};
+    const QVector3D col1{row0[1], row1[1], row2[1]};
+    const QVector3D col2{row0[2], row1[2], row2[2]};
+    const float scaleX = col0.length();
+    const float scaleY = col1.length();
+    const float scaleZ = col2.length();
+    return QVector3D(scaleX, scaleY, scaleZ);
+}
+
+QQuaternion QQuick3DInstancing::InstanceTableEntry::getRotation() const
+{
+    const QVector3D col0 = QVector3D(row0[0], row1[0], row2[0]).normalized();
+    const QVector3D col1 = QVector3D(row0[1], row1[1], row2[1]).normalized();
+    const QVector3D col2 = QVector3D(row0[2], row1[2], row2[2]).normalized();
+
+    const float data3x3[3*3] { // row-major order
+        col0[0], col1[0], col2[0],
+        col0[1], col1[1], col2[1],
+        col0[2], col1[2], col2[2],
+    };
+    QMatrix3x3 rot(data3x3);
+    return QQuaternion::fromRotationMatrix(rot).normalized();
+}
+
+QColor QQuick3DInstancing::InstanceTableEntry::getColor() const
+{
+    return QColor::fromRgbF(color[0], color[1], color[2], color[3]);
+}
+
+/*!
+    \qmlmethod vector3d QtQuick3D::Instancing::instancePosition(int index)
+    \since 6.3
+
+    Returns the position of the instance at \a index
+
+    \sa instanceScale, instanceRotation, instanceColor, instanceCustomData
+*/
+
+QVector3D QQuick3DInstancing::instancePosition(int index)
+{
+    auto *entry = getInstanceEntry(index);
+    if (!entry)
+        return {};
+
+    return QVector3D{ entry->row0[3], entry->row1[3], entry->row2[3] };
+}
+
+/*!
+    \qmlmethod vector3d QtQuick3D::Instancing::instanceScale(int index)
+    \since 6.3
+
+    Returns the scale of the instance at \a index
+
+    \sa instancePosition, instanceScale, instanceRotation, instanceColor, instanceCustomData
+*/
+
+QVector3D QQuick3DInstancing::instanceScale(int index)
+{
+    auto *entry = getInstanceEntry(index);
+    if (!entry)
+        return {};
+    return entry->getScale();
+}
+
+/*!
+    \qmlmethod quaternion QtQuick3D::Instancing::instanceRotation(int index)
+    \since 6.3
+
+    Returns a quaternion representing the rotation of the instance at \a index
+
+    \sa instancePosition, instanceScale, instanceRotation, instanceColor, instanceCustomData
+*/
+
+QQuaternion QQuick3DInstancing::instanceRotation(int index)
+{
+    const auto *entry = getInstanceEntry(index);
+    if (!entry)
+        return {};
+    return entry->getRotation();
+}
+
+/*!
+    \qmlmethod color QtQuick3D::Instancing::instanceColor(int index)
+    \since 6.3
+
+    Returns the color of the instance at \a index
+
+    \sa instancePosition, instanceScale, instanceRotation, instanceColor, instanceCustomData
+*/
+
+QColor QQuick3DInstancing::instanceColor(int index)
+{
+    const auto *entry = getInstanceEntry(index);
+    if (!entry)
+        return {};
+    return entry->getColor();
+}
+
+/*!
+    \qmlmethod vector3d QtQuick3D::Instancing::instanceCustomData(int index)
+    \since 6.3
+
+    Returns the custom data for the instance at \a index
+
+    \sa instancePosition, instanceScale, instanceRotation, instanceColor, instanceCustomData
+*/
+
+QVector4D QQuick3DInstancing::instanceCustomData(int index)
+{
+    const auto *entry = getInstanceEntry(index);
+    if (!entry)
+        return {};
+    return entry->instanceData;
+}
+
 void QQuick3DInstancing::setInstanceCountOverride(int instanceCountOverride)
 {
     Q_D(QQuick3DInstancing);
@@ -232,6 +336,7 @@ void QQuick3DInstancing::markDirty()
     Q_D(QQuick3DInstancing);
     d->dirty(QQuick3DObjectPrivate::DirtyType::Content);
     d->m_instanceDataChanged = true;
+    emit instanceTableChanged();
 }
 
 QSSGRenderGraphObject *QQuick3DInstancing::updateSpatialNode(QSSGRenderGraphObject *node)
@@ -423,6 +528,18 @@ QQmlListProperty<QQuick3DInstanceListEntry> QQuick3DInstanceList::instances()
                                                           qmlClearInstanceListEntries);
 }
 
+/*!
+    \qmlproperty int QtQuick3D::InstanceList::instanceCount
+    \since 6.3
+
+    This read-only property contains the number of instances in the list.
+*/
+
+int QQuick3DInstanceList::instanceCount() const
+{
+    return m_instances.size();
+}
+
 void QQuick3DInstanceList::onInstanceDestroyed(QObject *object)
 {
     if (m_instances.removeAll(object))
@@ -452,7 +569,7 @@ QQuick3DInstanceListEntry *QQuick3DInstanceList::qmlInstanceListEntryAt(QQmlList
 qsizetype QQuick3DInstanceList::qmlInstanceListEntriesCount(QQmlListProperty<QQuick3DInstanceListEntry> *list)
 {
     auto *self = static_cast<QQuick3DInstanceList *>(list->object);
-    return self->m_instances.count();
+    return self->m_instances.size();
 }
 
 void QQuick3DInstanceList::qmlClearInstanceListEntries(QQmlListProperty<QQuick3DInstanceListEntry> *list)
@@ -470,6 +587,7 @@ void QQuick3DInstanceList::handleInstanceChange()
 {
     m_dirty = true;
     markDirty();
+    emit instanceCountChanged();
 }
 
 void QQuick3DInstanceList::generateInstanceData()
@@ -668,6 +786,14 @@ void QQuick3DInstanceListEntry::setCustomData(QVector4D customData)
     Otherwise it is assumed to refer to an XML file. If an XML file \e{foo.xml} is specified, and
     the file \e{foo.xml.bin} exists, the binary file \e{foo.xml.bin} will be loaded instead.
 */
+
+/*!
+    \qmlproperty int QtQuick3D::FileInstancing::instanceCount
+    \since 6.3
+
+    This read-only property contains the number of instances in the instance table.
+*/
+
 static constexpr quint16 currentMajorVersion = 1;
 
 struct QQuick3DInstancingBinaryFileHeader
@@ -687,7 +813,7 @@ static bool writeInstanceTable(QIODevice *out, const QByteArray &instanceData, i
     header.offset = sizeof(header);
     header.count = instanceCount;
 
-    if (instanceData.size() != header.stride * instanceCount) {
+    if (instanceData.size() != qsizetype(header.stride) * instanceCount) {
         qWarning() << "inconsistent data";
         return false;
     }
@@ -775,7 +901,7 @@ bool QQuick3DFileInstancing::loadFromXmlFile(const QString &filename)
                     QVector3D scale { 1, 1, 1 };
                     for (auto &attr : reader.attributes()) {
                         if (attr.name() == QLatin1String("color")) {
-                            color = QColor(attr.value());
+                            color = QColor::fromString(attr.value());
                         } else if (attr.name() == QLatin1String("position")) {
                             position = toVector3D(attr.value());
                         } else if (attr.name() == QLatin1String("eulerRotation")) {
@@ -816,6 +942,11 @@ int QQuick3DFileInstancing::writeToBinaryFile(QIODevice *out)
     return success ? m_instanceCount : -1;
 }
 
+int QQuick3DFileInstancing::instanceCount() const
+{
+    return m_instanceCount;
+}
+
 bool QQuick3DFileInstancing::loadFromFile(const QUrl &source)
 {
     const QQmlContext *context = qmlContext(this);
@@ -827,10 +958,12 @@ bool QQuick3DFileInstancing::loadFromFile(const QUrl &source)
 
     const QString binaryFilePath = filePath + QStringLiteral(".bin");
 
-    if (loadFromBinaryFile(binaryFilePath))
-        return true;
+    int oldCount = m_instanceCount;
+    bool success = loadFromBinaryFile(binaryFilePath) || loadFromXmlFile(filePath);
+    if (m_instanceCount != oldCount)
+        emit instanceCountChanged();
 
-    return loadFromXmlFile(filePath);
+    return success;
 }
 
 QQuick3DFileInstancing::QQuick3DFileInstancing(QQuick3DObject *parent) : QQuick3DInstancing(parent) { }
