@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Quick 3D.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef QSSGPRINCIPLEDMATERIAL_H
 #define QSSGPRINCIPLEDMATERIAL_H
@@ -60,7 +34,7 @@ class Q_QUICK3D_EXPORT QQuick3DPrincipledMaterial : public QQuick3DMaterial
 
     Q_PROPERTY(float metalness READ metalness WRITE setMetalness NOTIFY metalnessChanged)
     Q_PROPERTY(QQuick3DTexture *metalnessMap READ metalnessMap WRITE setMetalnessMap NOTIFY metalnessMapChanged)
-    Q_PROPERTY(TextureChannelMapping metalnessChannel READ metalnessChannel WRITE setMetalnessChannel NOTIFY metalnessChannelChanged)
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping metalnessChannel READ metalnessChannel WRITE setMetalnessChannel NOTIFY metalnessChannelChanged)
 
     Q_PROPERTY(float specularAmount READ specularAmount WRITE setSpecularAmount NOTIFY specularAmountChanged)
     Q_PROPERTY(QQuick3DTexture *specularMap READ specularMap WRITE setSpecularMap NOTIFY specularMapChanged)
@@ -68,14 +42,14 @@ class Q_QUICK3D_EXPORT QQuick3DPrincipledMaterial : public QQuick3DMaterial
 
     Q_PROPERTY(float roughness READ roughness WRITE setRoughness NOTIFY roughnessChanged)
     Q_PROPERTY(QQuick3DTexture *roughnessMap READ roughnessMap WRITE setRoughnessMap NOTIFY roughnessMapChanged)
-    Q_PROPERTY(TextureChannelMapping roughnessChannel READ roughnessChannel WRITE setRoughnessChannel NOTIFY roughnessChannelChanged)
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping roughnessChannel READ roughnessChannel WRITE setRoughnessChannel NOTIFY roughnessChannelChanged)
 
     Q_PROPERTY(QVector3D emissiveFactor READ emissiveFactor WRITE setEmissiveFactor NOTIFY emissiveFactorChanged)
     Q_PROPERTY(QQuick3DTexture *emissiveMap READ emissiveMap WRITE setEmissiveMap NOTIFY emissiveMapChanged)
 
     Q_PROPERTY(float opacity READ opacity WRITE setOpacity NOTIFY opacityChanged)
     Q_PROPERTY(QQuick3DTexture *opacityMap READ opacityMap WRITE setOpacityMap NOTIFY opacityMapChanged)
-    Q_PROPERTY(TextureChannelMapping opacityChannel READ opacityChannel WRITE setOpacityChannel NOTIFY opacityChannelChanged)
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping opacityChannel READ opacityChannel WRITE setOpacityChannel NOTIFY opacityChannelChanged)
 
     Q_PROPERTY(QQuick3DTexture *normalMap READ normalMap WRITE setNormalMap NOTIFY normalMapChanged)
     Q_PROPERTY(float normalStrength READ normalStrength WRITE setNormalStrength NOTIFY normalStrengthChanged)
@@ -83,7 +57,7 @@ class Q_QUICK3D_EXPORT QQuick3DPrincipledMaterial : public QQuick3DMaterial
     Q_PROPERTY(QQuick3DTexture *specularReflectionMap READ specularReflectionMap WRITE setSpecularReflectionMap NOTIFY specularReflectionMapChanged)
 
     Q_PROPERTY(QQuick3DTexture *occlusionMap READ occlusionMap WRITE setOcclusionMap NOTIFY occlusionMapChanged)
-    Q_PROPERTY(TextureChannelMapping occlusionChannel READ occlusionChannel WRITE setOcclusionChannel NOTIFY occlusionChannelChanged)
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping occlusionChannel READ occlusionChannel WRITE setOcclusionChannel NOTIFY occlusionChannelChanged)
     Q_PROPERTY(float occlusionAmount READ occlusionAmount WRITE setOcclusionAmount NOTIFY occlusionAmountChanged)
 
     Q_PROPERTY(AlphaMode alphaMode READ alphaMode WRITE setAlphaMode NOTIFY alphaModeChanged)
@@ -93,11 +67,37 @@ class Q_QUICK3D_EXPORT QQuick3DPrincipledMaterial : public QQuick3DMaterial
     Q_PROPERTY(float lineWidth READ lineWidth WRITE setLineWidth NOTIFY lineWidthChanged)
 
     Q_PROPERTY(QQuick3DTexture *heightMap READ heightMap WRITE setHeightMap NOTIFY heightMapChanged REVISION(6, 2))
-    Q_PROPERTY(TextureChannelMapping heightChannel READ heightChannel WRITE setHeightChannel NOTIFY heightChannelChanged REVISION(6, 2))
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping heightChannel READ heightChannel WRITE setHeightChannel NOTIFY heightChannelChanged REVISION(6, 2))
     Q_PROPERTY(float heightAmount READ heightAmount WRITE setHeightAmount NOTIFY heightAmountChanged REVISION(6, 2))
     Q_PROPERTY(int minHeightMapSamples READ minHeightMapSamples WRITE setMinHeightMapSamples NOTIFY minHeightMapSamplesChanged REVISION(6, 2))
     Q_PROPERTY(int maxHeightMapSamples READ maxHeightMapSamples WRITE setMaxHeightMapSamples NOTIFY maxHeightMapSamplesChanged REVISION(6, 2))
 
+    Q_PROPERTY(float clearcoatAmount READ clearcoatAmount WRITE setClearcoatAmount NOTIFY clearcoatAmountChanged REVISION(6, 3))
+    Q_PROPERTY(QQuick3DTexture *clearcoatMap READ clearcoatMap WRITE setClearcoatMap NOTIFY clearcoatMapChanged REVISION(6, 3))
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping clearcoatChannel READ clearcoatChannel WRITE setClearcoatChannel NOTIFY
+                       clearcoatChannelChanged REVISION(6, 3))
+    Q_PROPERTY(float clearcoatRoughnessAmount READ clearcoatRoughnessAmount WRITE setClearcoatRoughnessAmount NOTIFY
+                       clearcoatRoughnessAmountChanged REVISION(6, 3))
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping clearcoatRoughnessChannel READ clearcoatRoughnessChannel WRITE
+                       setClearcoatRoughnessChannel NOTIFY clearcoatRoughnessChannelChanged REVISION(6, 3))
+    Q_PROPERTY(QQuick3DTexture *clearcoatRoughnessMap READ clearcoatRoughnessMap WRITE setClearcoatRoughnessMap NOTIFY
+                       clearcoatRoughnessMapChanged REVISION(6, 3))
+    Q_PROPERTY(QQuick3DTexture *clearcoatNormalMap READ clearcoatNormalMap WRITE setClearcoatNormalMap NOTIFY
+                       clearcoatNormalMapChanged REVISION(6, 3))
+
+    Q_PROPERTY(float transmissionFactor READ transmissionFactor WRITE setTransmissionFactor NOTIFY transmissionFactorChanged)
+    Q_PROPERTY(QQuick3DTexture * transmissionMap READ transmissionMap WRITE setTransmissionMap NOTIFY transmissionMapChanged)
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping transmissionChannel READ transmissionChannel WRITE setTransmissionChannel NOTIFY transmissionChannelChanged)
+
+    Q_PROPERTY(float thicknessFactor READ thicknessFactor WRITE setThicknessFactor NOTIFY thicknessFactorChanged REVISION(6, 3))
+    Q_PROPERTY(QQuick3DTexture *thicknessMap READ thicknessMap WRITE setThicknessMap NOTIFY thicknessMapChanged REVISION(6, 3))
+    Q_PROPERTY(QQuick3DMaterial::TextureChannelMapping thicknessChannel READ thicknessChannel WRITE setThicknessChannel NOTIFY
+                       thicknessChannelChanged REVISION(6, 3))
+    Q_PROPERTY(float attenuationDistance READ attenuationDistance WRITE setAttenuationDistance NOTIFY attenuationDistanceChanged REVISION(6, 3))
+    Q_PROPERTY(QColor attenuationColor READ attenuationColor WRITE setAttenuationColor NOTIFY attenuationColorChanged REVISION(6, 3))
+
+    Q_PROPERTY(float indexOfRefraction READ indexOfRefraction WRITE setIndexOfRefraction NOTIFY indexOfRefractionChanged REVISION(6, 3))
+
     QML_NAMED_ELEMENT(PrincipledMaterial)
 
 public:
@@ -159,6 +159,26 @@ public:
     Q_REVISION(6, 2) int minHeightMapSamples() const;
     Q_REVISION(6, 2) int maxHeightMapSamples() const;
 
+    Q_REVISION(6, 3) float clearcoatAmount() const;
+    Q_REVISION(6, 3) QQuick3DTexture *clearcoatMap() const;
+    Q_REVISION(6, 3) TextureChannelMapping clearcoatChannel() const;
+    Q_REVISION(6, 3) float clearcoatRoughnessAmount() const;
+    Q_REVISION(6, 3) TextureChannelMapping clearcoatRoughnessChannel() const;
+    Q_REVISION(6, 3) QQuick3DTexture *clearcoatRoughnessMap() const;
+    Q_REVISION(6, 3) QQuick3DTexture *clearcoatNormalMap() const;
+
+    Q_REVISION(6, 3) float transmissionFactor() const;
+    Q_REVISION(6, 3) QQuick3DTexture *transmissionMap() const;
+    Q_REVISION(6, 3) TextureChannelMapping transmissionChannel() const;
+
+    Q_REVISION(6, 3) float thicknessFactor() const;
+    Q_REVISION(6, 3) QQuick3DTexture *thicknessMap() const;
+    Q_REVISION(6, 3) const TextureChannelMapping &thicknessChannel() const;
+    Q_REVISION(6, 3) float attenuationDistance() const;
+    Q_REVISION(6, 3) const QColor &attenuationColor() const;
+
+    Q_REVISION(6, 3) float indexOfRefraction() const;
+
 public Q_SLOTS:
     void setLighting(QQuick3DPrincipledMaterial::Lighting lighting);
     void setBlendMode(QQuick3DPrincipledMaterial::BlendMode blendMode);
@@ -189,11 +209,31 @@ public Q_SLOTS:
     void setPointSize(float size);
     void setLineWidth(float width);
     Q_REVISION(6, 2) void setHeightMap(QQuick3DTexture *heightMap);
-    Q_REVISION(6, 2) void setHeightChannel(TextureChannelMapping channel);
+    Q_REVISION(6, 2) void setHeightChannel(QQuick3DMaterial::TextureChannelMapping channel);
     Q_REVISION(6, 2) void setHeightAmount(float heightAmount);
     Q_REVISION(6, 2) void setMinHeightMapSamples(int samples);
     Q_REVISION(6, 2) void setMaxHeightMapSamples(int samples);
 
+    Q_REVISION(6, 3) void setClearcoatAmount(float newClearcoatAmount);
+    Q_REVISION(6, 3) void setClearcoatMap(QQuick3DTexture *newClearcoatMap);
+    Q_REVISION(6, 3) void setClearcoatChannel(QQuick3DMaterial::TextureChannelMapping newClearcoatChannel);
+    Q_REVISION(6, 3) void setClearcoatRoughnessAmount(float newClearcoatRoughnessAmount);
+    Q_REVISION(6, 3) void setClearcoatRoughnessChannel(QQuick3DMaterial::TextureChannelMapping newClearcoatRoughnessChannel);
+    Q_REVISION(6, 3) void setClearcoatRoughnessMap(QQuick3DTexture *newClearcoatRoughnessMap);
+    Q_REVISION(6, 3) void setClearcoatNormalMap(QQuick3DTexture *newClearcoatNormalMap);
+
+    Q_REVISION(6, 3) void setTransmissionFactor(float newTransmissionFactor);
+    Q_REVISION(6, 3) void setTransmissionMap(QQuick3DTexture *newTransmissionMap);
+    Q_REVISION(6, 3) void setTransmissionChannel(QQuick3DMaterial::TextureChannelMapping newTransmissionChannel);
+
+    Q_REVISION(6, 3) void setThicknessFactor(float newThicknessFactor);
+    Q_REVISION(6, 3) void setThicknessMap(QQuick3DTexture *newThicknessMap);
+    Q_REVISION(6, 3) void setThicknessChannel(const QQuick3DMaterial::TextureChannelMapping &newThicknessChannel);
+    Q_REVISION(6, 3) void setAttenuationDistance(float newAttenuationDistance);
+    Q_REVISION(6, 3) void setAttenuationColor(const QColor &newAttenuationColor);
+
+    Q_REVISION(6, 3) void setIndexOfRefraction(float indexOfRefraction);
+
 Q_SIGNALS:
     void lightingChanged(QQuick3DPrincipledMaterial::Lighting lighting);
     void blendModeChanged(QQuick3DPrincipledMaterial::BlendMode blendMode);
@@ -224,11 +264,31 @@ Q_SIGNALS:
     void pointSizeChanged();
     void lineWidthChanged();
     Q_REVISION(6, 2) void heightMapChanged(QQuick3DTexture *heightMap);
-    Q_REVISION(6, 2) void heightChannelChanged(TextureChannelMapping channel);
+    Q_REVISION(6, 2) void heightChannelChanged(QQuick3DMaterial::TextureChannelMapping channel);
     Q_REVISION(6, 2) void heightAmountChanged(float heightAmount);
     Q_REVISION(6, 2) void minHeightMapSamplesChanged(int samples);
     Q_REVISION(6, 2) void maxHeightMapSamplesChanged(int samples);
 
+    Q_REVISION(6, 3) void clearcoatAmountChanged(float amount);
+    Q_REVISION(6, 3) void clearcoatMapChanged(QQuick3DTexture *texture);
+    Q_REVISION(6, 3) void clearcoatChannelChanged(QQuick3DMaterial::TextureChannelMapping channel);
+    Q_REVISION(6, 3) void clearcoatRoughnessAmountChanged(float amount);
+    Q_REVISION(6, 3) void clearcoatRoughnessChannelChanged(QQuick3DMaterial::TextureChannelMapping channel);
+    Q_REVISION(6, 3) void clearcoatRoughnessMapChanged(QQuick3DTexture *texture);
+    Q_REVISION(6, 3) void clearcoatNormalMapChanged(QQuick3DTexture *texture);
+
+    Q_REVISION(6, 3) void transmissionFactorChanged(float amount);
+    Q_REVISION(6, 3) void transmissionMapChanged(QQuick3DTexture *texture);
+    Q_REVISION(6, 3) void transmissionChannelChanged(QQuick3DMaterial::TextureChannelMapping channel);
+
+    Q_REVISION(6, 3) void thicknessFactorChanged(float amount);
+    Q_REVISION(6, 3) void thicknessMapChanged(QQuick3DTexture *texture);
+    Q_REVISION(6, 3) void thicknessChannelChanged(QQuick3DMaterial::TextureChannelMapping channel);
+    Q_REVISION(6, 3) void attenuationDistanceChanged(float distance);
+    Q_REVISION(6, 3) void attenuationColorChanged(QColor color);
+
+    Q_REVISION(6, 3) void indexOfRefractionChanged(float indexOfRefraction);
+
 protected:
     QSSGRenderGraphObject *updateSpatialNode(QSSGRenderGraphObject *node) override;
     void markAllDirty() override;
@@ -250,7 +310,10 @@ private:
         AlphaModeDirty = 0x00000400,
         PointSizeDirty = 0x00000800,
         LineWidthDirty = 0x00001000,
-        HeightDirty = 0x00002000
+        HeightDirty = 0x00002000,
+        ClearcoatDirty = 0x00004000,
+        TransmissionDirty = 0x00008000,
+        VolumeDirty = 0x00010000
     };
 
     void updateSceneManager(QQuick3DSceneManager *window);
@@ -277,7 +340,7 @@ private:
     QQuick3DTexture *m_metalnessMap = nullptr;
     QQuick3DTexture *m_occlusionMap = nullptr;
     float m_specularTint = 0.0f;
-    float m_specularAmount = 0.5f;
+    float m_specularAmount = 1.0f;
     float m_roughness = 0.0f;
     float m_opacity = 1.0f;
     float m_metalnessAmount = 0.0f;
@@ -295,6 +358,22 @@ private:
     float m_heightAmount = 0.0f;
     int m_minHeightMapSamples = 8;
     int m_maxHeightMapSamples = 32;
+    float m_clearcoatAmount = 0.0f;
+    QQuick3DTexture *m_clearcoatMap = nullptr;
+    TextureChannelMapping m_clearcoatChannel = QQuick3DMaterial::R;
+    float m_clearcoatRoughnessAmount = 0.0f;
+    TextureChannelMapping m_clearcoatRoughnessChannel = QQuick3DMaterial::G;
+    QQuick3DTexture *m_clearcoatRoughnessMap = nullptr;
+    QQuick3DTexture *m_clearcoatNormalMap = nullptr;
+    float m_transmissionFactor = 0.0f;
+    QQuick3DTexture *m_transmissionMap = nullptr;
+    TextureChannelMapping m_transmissionChannel = QQuick3DMaterial::R;
+    float m_thicknessFactor = 0.0f;
+    QQuick3DTexture *m_thicknessMap = nullptr;
+    TextureChannelMapping m_thicknessChannel = QQuick3DMaterial::G;
+    float m_attenuationDistance = std::numeric_limits<float>::infinity();
+    QColor m_attenuationColor = Qt::white;
+    float m_indexOfRefraction = 1.5f;
 
     quint32 m_dirtyAttributes = 0xffffffff; // all dirty by default
     void markDirty(DirtyType type);
